use std::{error::Error as StdError, fmt::Display};

use axum::{
    response::{IntoResponse, Response},
    Json,
};
use http::StatusCode;
use serde::Serialize;

#[derive(Serialize)]
pub struct Reply<T>
where
    T: Serialize,
{
    pub code: i32,
    pub err: bool,
    pub msg: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub data: Option<T>,
}

pub struct ResData<T>(pub Option<T>)
where
    T: Serialize;

impl<T> IntoResponse for ResData<T>
where
    T: Serialize,
{
    fn into_response(self) -> Response {
        let ResData(data) = self;
        let reply = Reply {
            code: 0,
            err: false,
            msg: String::from("OK"),
            data,
        };

        Json(reply).into_response()
    }
}

impl<T: Serialize> ResData<T> {
    pub fn none() -> Self {
        ResData(None)
    }
    pub fn some(data: T) -> Self {
        ResData(Some(data))
    }
}

#[derive(Debug)]
pub enum ResErr {
    Error(i32, String),
    ErrParams(Option<String>),
    ErrAuth(Option<String>),
    ErrPerm(Option<String>),
    ErrNotFound(Option<String>),
    ErrService(Option<String>),
    ErrSocial(Option<String>),
    ErrAccountDisable(Option<String>),
    ErrSqlx(Option<String>),
}

use ResErr::*;

impl IntoResponse for ResErr {
    fn into_response(self) -> Response {
        let status = match self {
            Error(code, msg) => (StatusCode::INTERNAL_SERVER_ERROR, code, msg),
            ErrParams(msg) => {
                let code = 10000;

                match msg {
                    Some(v) => (StatusCode::BAD_REQUEST, code, v),
                    None => (StatusCode::BAD_REQUEST, code, String::from("参数错误")),
                }
            }
            ErrAuth(msg) => {
                let code = 20000;

                match msg {
                    Some(v) => (StatusCode::UNAUTHORIZED, code, v),
                    None => (
                        StatusCode::UNAUTHORIZED,
                        code,
                        String::from("未授权，请先登录"),
                    ),
                }
            }
            ErrPerm(msg) => {
                let code = 30000;

                match msg {
                    Some(v) => (StatusCode::FORBIDDEN, code, v),
                    None => (StatusCode::FORBIDDEN, code, String::from("权限不足")),
                }
            }
            ErrNotFound(msg) => {
                let code = 40000;

                match msg {
                    Some(v) => (StatusCode::NOT_FOUND, code, v),
                    None => (StatusCode::NOT_FOUND, code, String::from("数据不存在")),
                }
            }
            ErrService(msg) => {
                let code = 50000;

                match msg {
                    Some(v) => (StatusCode::INTERNAL_SERVER_ERROR, code, v),
                    None => (
                        StatusCode::INTERNAL_SERVER_ERROR,
                        code,
                        String::from("内部服务器错误，请稍后重试"),
                    ),
                }
            }
            ErrSocial(msg) => {
                let code = 80000;
                match msg {
                    Some(v) => (StatusCode::INTERNAL_SERVER_ERROR, code, v),
                    None => (
                        StatusCode::INTERNAL_SERVER_ERROR,
                        code,
                        String::from("社交业务异常"),
                    ),
                }
            }
            ErrAccountDisable(msg) => {
                let code = 90000;
                match msg {
                    Some(v) => (StatusCode::FORBIDDEN, code, v),
                    None => (StatusCode::FORBIDDEN, code, String::from("账户已禁用")),
                }
            }
            ErrSqlx(msg) => {
                let code = 90000;
                match msg {
                    Some(v) => (StatusCode::INTERNAL_SERVER_ERROR, code, v),
                    None => (
                        StatusCode::INTERNAL_SERVER_ERROR,
                        code,
                        String::from("业务错误"),
                    ),
                }
            }
        };

        let reply: Reply<()> = Reply {
            code: status.1,
            err: false,
            msg: status.2,
            data: None,
        };
        (status.0, Json(reply)).into_response()
    }
}

impl Display for ResErr {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        // write!(f, "{:?}", self.message)
        match self {
            Error(value, message) => write!(f, "{} - {:?}", value, message),
            ErrParams(message)
            | ErrAuth(message)
            | ErrPerm(message)
            | ErrNotFound(message)
            | ErrService(message)
            | ErrSocial(message)
            | ErrAccountDisable(message)
            | ErrSqlx(message) => write!(f, "{:?}", message),
        }
    }
}

impl StdError for ResErr {}

impl From<String> for ResErr {
    fn from(value: String) -> Self {
        tracing::error!("服务错误：{}", value);
        ErrService(Some(value))
    }
}

impl From<&str> for ResErr {
    fn from(value: &str) -> Self {
        tracing::error!("服务错误：{}", value);
        ResErr::from(value.to_string())
    }
}

impl From<sqlx::Error> for ResErr {
    fn from(value: sqlx::Error) -> Self {
        tracing::error!("数据库操作错误：{}", value);
        ErrSqlx(Some(value.to_string()))
    }
}

impl From<validator::ValidationErrors> for ResErr {
    fn from(value: validator::ValidationErrors) -> Self {
        tracing::error!("数据校验错误：{}", value);
        ErrParams(Some(value.to_string()))
    }
}

impl From<std::io::Error> for ResErr {
    fn from(value: std::io::Error) -> Self {
        tracing::error!("IO错误：{}", value);
        ErrService(Some(value.to_string()))
    }
}

impl From<tokio::task::JoinError> for ResErr {
    fn from(value: tokio::task::JoinError) -> Self {
        tracing::error!("线程池错误：{}", value);
        ErrService(Some(value.to_string()))
    }
}

impl Default for ResErr {
    fn default() -> Self {
        ErrService(None)
    }
}

impl ResErr {
    #[inline]
    pub fn error<T>(code: i32, msg: T) -> Self
    where
        T: Into<String>,
    {
        Error(code, msg.into())
    }

    #[inline]
    pub fn params<T>(msg: T) -> Self
    where
        T: Into<String>,
    {
        ErrParams(Some(msg.into()))
    }

    #[inline]
    pub fn auth<T>(msg: T) -> Self
    where
        T: Into<String>,
    {
        ErrAuth(Some(msg.into()))
    }

    #[inline]
    pub fn perm<T>(msg: T) -> Self
    where
        T: Into<String>,
    {
        ErrPerm(Some(msg.into()))
    }

    #[inline]
    pub fn not_found<T>(msg: T) -> Self
    where
        T: Into<String>,
    {
        ErrNotFound(Some(msg.into()))
    }

    #[inline]
    pub fn service<T>(msg: T) -> Self
    where
        T: Into<String>,
    {
        ErrService(Some(msg.into()))
    }

    #[inline]
    pub fn social<T>(msg: T) -> Self
    where
        T: Into<String>,
    {
        ErrSocial(Some(msg.into()))
    }
}

pub type ResResult<T> = Result<T, ResErr>;
